/* $Id$ -*- C++ -*-
 */

#include "contour.h"
#include <cstring>

const double DTOR = M_PI / 180;

enum MODE {mUNKNOWN, mINTERFERENCE, mCOMMON};

double distance(const Station& s1, const Station& s2)
{
  double R = 6371; // km
  double dLat = s2._latitude - s1._latitude;
  double dLon = s2._longitude - s1._longitude;

  double a = sin(dLat/2) * sin(dLat/2) +
    cos(s1._latitude) * cos(s2._latitude) * sin(dLon/2) * sin(dLon/2); 
  double c = 2 * atan2(sqrt(a), sqrt(1-a)); 
  return R * c;
}

Angle angle(const Station& s1, const Station& s2)
{
  double dLon = s2._longitude - s1._longitude;

  double y = sin(dLon) * cos(s2._latitude);
  double x = cos(s1._latitude)*sin(s2._latitude)-sin(s1._latitude)*cos(s2._latitude)*cos(dLon);
  return atan2(y, x)/DTOR;
}

void do_it(Station& s1, Station& s2, MODE mode)
{
  double distance = ::distance(s1,s2);
  Angle angle = ::angle(s1, s2);

  std::string relation;
  switch (std::abs(s1._channel - s2._channel)) {
  case 0:  relation = "co-channel";      break;
  case 1:  relation = "1st-adjacent";    break;
  case 2:  relation = "2nd-adjacent";    break;
  case 3:  relation = "3rd-adjacent";    break;
  default: relation = "no-interference"; break;
  };

  bool interfering_is_smaller = false;
  {
    Radial r1 = s1[angle];
    Radial r2 = s2[angle - 180];
    double protected_raw = r1._p60;
    double interfering_raw;
    switch (std::abs(s1._channel - s2._channel)) {
    case 0:  interfering_raw = r2._i40;  break;
    case 1:  interfering_raw = r2._i54;  break;
    case 2:  interfering_raw = r2._i100; break;
    case 3:  interfering_raw = r2._i100; break;
    default: interfering_raw = 0.;	 break;
    };
    interfering_is_smaller = (interfering_raw < protected_raw);
  }
  if (interfering_is_smaller) {
    std::cout << "---";
  }else{
  }
  std::cout << relation << ", distance=" << distance << " bearing from a to b =" << angle << '\n';
  if (mode != mINTERFERENCE || std::abs(s1._channel - s2._channel) <= 3) {
    std::cout << "bear-ab\tbear-ba\ta-prot\tb-int\tap-prot\tbp-int\tmargin\n";
    for (double offset = -85; offset <= 85; offset += 5) {
      Angle a1, a2;
      Radial r1, r2;
      double offset1, offset2;
      double protected_raw, interfering_raw;
      if (mode == mCOMMON) {
	offset2 = offset;
	a2 = angle + 180 - offset2;
	r2 = s2[a2];
	interfering_raw = r2._p60;
	double ref = interfering_raw;
	offset1 = atan2(ref * sin(offset * DTOR), distance - ref * cos(offset * DTOR)) / DTOR;
	a1 = angle + offset1;
	r1 = s1[a1];
	protected_raw = r1._p60;
      }else if (interfering_is_smaller) {
	offset2 = offset;
	a2 = angle + 180 - offset2;
	r2 = s2[a2];
	switch (std::abs(s1._channel - s2._channel)) {
	case 0:  interfering_raw = r2._i40;  break;
	case 1:  interfering_raw = r2._i54;  break;
	case 2:  interfering_raw = r2._i100; break;
	case 3:  interfering_raw = r2._i100; break;
	default: interfering_raw = 0.;	   break;
	};
	double ref = interfering_raw;
	offset1 = atan2(ref * sin(offset * DTOR), distance - ref * cos(offset * DTOR)) / DTOR;
	a1 = angle + offset1;
	r1 = s1[a1];
	protected_raw = r1._p60;
      }else{
	offset1 = offset;
	a1 = angle + offset1;
	r1 = s1[a1];
	protected_raw = r1._p60;
	double ref = protected_raw;
	offset2 = atan2(ref * sin(offset * DTOR), distance - ref * cos(offset * DTOR)) / DTOR;
	a2 = angle + 180 - offset2;
	r2 = s2[a2];
	switch (std::abs(s1._channel - s2._channel)) {
	case 0:  interfering_raw = r2._i40;  break;
	case 1:  interfering_raw = r2._i54;  break;
	case 2:  interfering_raw = r2._i100; break;
	case 3:  interfering_raw = r2._i100; break;
	default: interfering_raw = 0.;	   break;
	};
      }
      
      double protected_projected   = protected_raw   * cos(offset1  * DTOR);
      double interfering_projected = interfering_raw * cos(offset2 * DTOR);
      
      std::cout.precision(4);
      std::cout <<         a1
		<< '\t' << a2
		<< '\t' << protected_raw
		<< '\t' << interfering_raw
		<< '\t' << protected_projected
		<< '\t' << interfering_projected
		<< '\t' << distance - (protected_projected + interfering_projected)
		<< "\n";
    }
  }else{
  }
}


int main(int argc, char* argv[])
{
  int file_arg = 2;
  MODE mode = mUNKNOWN;
  if (argc <= 1) {
    std::cerr << "no args?\n";
    exit(1);
  }else if (strncmp("-i", argv[1], 2)==0) {
    std::cout << "interference\n";
    mode = mINTERFERENCE;
  }else if (strncmp("-c", argv[1], 2)==0) {
    std::cout << "common\n";
    mode = mCOMMON;
  }else{
    std::cout << "????\n";
    file_arg = 1;
  }
  
  if (argc <= file_arg) {
    std::cerr << "no file?\n";
    exit(1);
  }
  
  std::cout << argv[file_arg] << "---" << file_arg << '\n';
  std::ifstream f1(argv[file_arg]);
  if (!f1) {
    std::cerr << "can't open file: " << argv[file_arg] << '\n';
    exit(1);
  }
  Station s1(f1);

  for (int ii = file_arg+1; ii < argc; ++ii) {
    if (strcmp(argv[ii],  argv[file_arg]) == 0) {
      // silently skip comparing to itself
      continue;
    }
    std::ifstream f2(argv[ii]);
    if (!f2) {
      std::cerr << "can't open file: " << argv[ii] << ", skipping\n";
      continue;
    }
    Station s2(f2);

    std::cout << "margin for overlap to " << argv[file_arg] << " from " << argv[ii] << '\n';
    do_it(s1, s2, mode);
    std::cout << '\n';
    
    std::cout << "margin for overlap to " << argv[ii] << " from " << argv[file_arg] << '\n';
    do_it(s2, s1, mode);
    std::cout << '\n';
  }

  return 0;
}
